@@ -33,7 +33,7 @@ get_feed_args(struct menu_state *ctx, size_t index)
 static unsigned
 paint_feed(struct menu_state *ctx, size_t index)
 {
-	if (ctx->feeds[index]->has_errors && !get_cfg_bool(&ctx->feeds[index]->cfg, CFG_SUPPRESS_ERRORS)) {
+	if (ctx->feeds[index]->errors->len > 0 && !get_cfg_bool(&ctx->feeds[index]->cfg, CFG_SUPPRESS_ERRORS)) {
 		return get_cfg_color(&ctx->feeds[index]->cfg, CFG_COLOR_LIST_FEED_FAILED);
 	} else if (ctx->feeds[index]->unread_count > 0) {
 		return get_cfg_color(&ctx->feeds[index]->cfg, CFG_COLOR_LIST_FEED_UNREAD);
@@ -48,6 +48,12 @@ is_feed_unread(struct menu_state *ctx, size_t index)
 	return ctx->feeds[index]->unread_count > 0;
 }
 
+static bool
+is_feed_failed(struct menu_state *ctx, size_t index)
+{
+	return ctx->feeds[index]->errors->len > 0;
+}
+
 static int
 compare_feeds_initial(const void *data1, const void *data2)
 {
@@ -110,6 +116,7 @@ feeds_menu_loop(struct menu_state *m)
 	m->get_args     = &get_feed_args;
 	m->paint_action = &paint_feed;
 	m->unread_state = &is_feed_unread;
+	m->failed_state = &is_feed_failed;
 	m->write_action = &list_menu_writer;
 	m->entry_format = get_cfg_wstring(NULL, CFG_MENU_FEED_ENTRY_FORMAT);
 	if (m->feeds_count < 1) {
@@ -118,21 +125,21 @@ feeds_menu_loop(struct menu_state *m)
 	} else if (!(m->flags & MENU_DISABLE_SETTINGS)) {
 		// Don't set the menu names here because it's redundant!
 		if (get_cfg_bool(NULL, CFG_FEEDS_MENU_PARAMOUNT_EXPLORE) && db_count_items(m->feeds_original, m->feeds_count, false)) {
-			return setup_menu(&items_menu_loop, NULL, m->feeds_original, m->feeds_count, MENU_IS_EXPLORE);
+			return setup_menu(&items_menu_loop, NULL, m->feeds_original, m->feeds_count, MENU_IS_EXPLORE, NULL);
 		} else if (m->feeds_count == 1 && db_count_items(m->feeds_original, m->feeds_count, false)) {
-			return setup_menu(&items_menu_loop, NULL, m->feeds_original, m->feeds_count, MENU_SWALLOW);
+			return setup_menu(&items_menu_loop, NULL, m->feeds_original, m->feeds_count, MENU_SWALLOW, NULL);
 		}
 	}
 	if (m->is_initialized == false) {
-		m->feeds = malloc(sizeof(struct feed_entry *) * m->feeds_count);
+		m->feeds = newsraft_malloc(sizeof(struct feed_entry *) * m->feeds_count);
 		memcpy(m->feeds, m->feeds_original, sizeof(struct feed_entry *) * m->feeds_count);
 		sort_feeds(m, get_sorting_id(get_cfg_string(NULL, CFG_MENU_FEED_SORTING)->ptr), false);
 	}
 	start_menu();
-	const struct wstring *macro;
+	const struct wstring *arg;
 	while (true) {
-		input_id cmd = get_input(m->feeds[m->view_sel]->binds, NULL, &macro);
-		if (handle_list_menu_control(m, cmd, macro) == true) {
+		input_id cmd = get_input(m->feeds[m->view_sel]->binds, NULL, &arg);
+		if (handle_list_menu_control(m, cmd, arg) == true) {
 			continue;
 		}
 		switch (cmd) {
@@ -144,11 +151,11 @@ feeds_menu_loop(struct menu_state *m)
 			case INPUT_RELOAD_ALL:      queue_updates(m->feeds_original, m->feeds_count);  break;
 			case INPUT_QUIT_HARD:       return NULL;
 			case INPUT_ENTER:
-				return setup_menu(&items_menu_loop, NULL, m->feeds + m->view_sel, 1, MENU_NORMAL);
+				return setup_menu(&items_menu_loop, NULL, m->feeds + m->view_sel, 1, MENU_NORMAL, NULL);
 			case INPUT_TOGGLE_EXPLORE_MODE:
-				return setup_menu(&items_menu_loop, NULL, m->feeds, m->feeds_count, MENU_IS_EXPLORE);
+				return setup_menu(&items_menu_loop, NULL, m->feeds, m->feeds_count, MENU_IS_EXPLORE, NULL);
 			case INPUT_APPLY_SEARCH_MODE_FILTER:
-				return setup_menu(&items_menu_loop, NULL, m->feeds, m->feeds_count, MENU_IS_EXPLORE | MENU_USE_SEARCH);
+				return setup_menu(&items_menu_loop, NULL, m->feeds, m->feeds_count, MENU_IS_EXPLORE, NULL);
 			case INPUT_NAVIGATE_BACK:
 				if (get_menu_depth() < 2) break;
 				// fall through
@@ -163,8 +170,13 @@ feeds_menu_loop(struct menu_state *m)
 			case INPUT_SORT_BY_ALPHABET:
 				sort_feeds(m, feeds_sort == SORT_BY_ALPHABET_ASC ? SORT_BY_ALPHABET_DESC : SORT_BY_ALPHABET_ASC, true);
 				break;
+			case INPUT_FIND_COMMAND:
+				return setup_menu(&items_menu_loop, NULL, m->feeds_original, m->feeds_count, MENU_IS_EXPLORE, arg);
+			case INPUT_DATABASE_COMMAND:
+				db_perform_user_edit(arg, m->feeds + m->view_sel, 1, NULL);
+				break;
 			case INPUT_VIEW_ERRORS:
-				return setup_menu(&errors_pager_loop, NULL, m->feeds + m->view_sel, 1, MENU_NORMAL);
+				return setup_menu(&errors_pager_loop, NULL, m->feeds + m->view_sel, 1, MENU_NORMAL, NULL);
 		}
 	}
 	return NULL;
