@@ -36,16 +36,13 @@ inserter_worker(void *dummy)
 		}
 
 		struct feed_entry *feed = target->feed_entry;
-		bool had_errors = feed->has_errors;
 
 		if (target->is_failed) {
-			feed->has_errors = true;
+			str_appendf(target->new_errors, "Feed update failed!\n");
 		} else {
 			if (target->is_canceled == false) {
-				if (insert_feed(feed, &target->feed)) {
-					feed->has_errors = false;
-				} else {
-					feed->has_errors = true;
+				if (!insert_feed(feed, &target->feed)) {
+					str_appendf(target->new_errors, "Feed insertion failed!\n");
 				}
 			} else {
 				db_update_feed_int64(feed->link, "update_date", feed->update_date, true);
@@ -59,19 +56,32 @@ inserter_worker(void *dummy)
 					db_update_feed_int64(feed->link,  "http_header_last_modified", target->feed.http_header_last_modified, true);
 					db_update_feed_int64(feed->link,  "http_header_expires", target->feed.http_header_expires, true);
 				}
-
-				// This feed is successfully canceled - no need for error indication.
-				feed->has_errors = false;
 			}
 		}
 
+		// Interface routines actively access feed's name and errors strings in other threads.
+		// The lock is here to prevent race condition which can potentially lead to segmentation fault.
+		// TODO: make it less ugly perhaps?
+		pthread_mutex_lock(&interface_lock);
+
 		bool need_redraw = false;
 
-		if (feed->has_errors != had_errors) {
-			need_redraw = true;
+		if (target->new_errors->len == 0) {
+			if (feed->errors->len > 0) {
+				need_redraw = true;
+			}
+			empty_string(feed->errors);
+		} else {
+			if (feed->errors->len == 0) {
+				need_redraw = true;
+			}
+			catss(feed->errors, target->new_errors);
 		}
 
-		int64_t new_unread_count = get_unread_items_count_of_the_feed(feed->link);
+		size_t errors_len_before_count = feed->errors->len;
+		int64_t new_unread_count = db_count_items(&feed, 1, true);
+		size_t errors_len_after_count = feed->errors->len;
+
 		if (new_unread_count >= 0 && new_unread_count != feed->unread_count) {
 			if (new_unread_count > feed->unread_count) {
 				target->new_items_count = new_unread_count - feed->unread_count;
@@ -79,11 +89,10 @@ inserter_worker(void *dummy)
 			feed->unread_count = new_unread_count;
 			need_redraw = true;
 		}
+		if (errors_len_before_count != errors_len_after_count) {
+			need_redraw = true;
+		}
 
-		// Interface routines actively access the feed name string in other threads.
-		// The lock is here to prevent race condition which can potentially lead to segmentation fault.
-		// TODO: make it less ugly perhaps?
-		pthread_mutex_lock(&interface_lock);
 		if (STRING_IS_EMPTY(feed->name)) {
 			struct string *title = db_get_string_from_feed_table(feed->link, "title", 5);
 			if (title != NULL) {
@@ -93,6 +102,7 @@ inserter_worker(void *dummy)
 				need_redraw = true;
 			}
 		}
+
 		pthread_mutex_unlock(&interface_lock);
 
 		if (curses_is_running() && need_redraw == true) {
