@@ -6,58 +6,45 @@ static inline bool
 append_sorting_order_expression_to_query(struct string *q, int order)
 {
 	switch (order) {
-		case SORT_BY_TIME_ASC:       return catas(q, " ORDER BY MAX(publication_date, update_date) ASC, rowid ASC", 59);
-		case SORT_BY_TIME_DESC:      return catas(q, " ORDER BY MAX(publication_date, update_date) DESC, rowid DESC", 61);
-		case SORT_BY_ROWID_ASC:      return catas(q, " ORDER BY rowid ASC", 19);
-		case SORT_BY_ROWID_DESC:     return catas(q, " ORDER BY rowid DESC", 20);
-		case SORT_BY_UNREAD_ASC:     return catas(q, " ORDER BY unread ASC, MAX(publication_date, update_date) DESC, rowid DESC", 73);
-		case SORT_BY_UNREAD_DESC:    return catas(q, " ORDER BY unread DESC, MAX(publication_date, update_date) DESC, rowid DESC", 74);
-		case SORT_BY_IMPORTANT_ASC:  return catas(q, " ORDER BY important ASC, MAX(publication_date, update_date) DESC, rowid DESC", 76);
-		case SORT_BY_IMPORTANT_DESC: return catas(q, " ORDER BY important DESC, MAX(publication_date, update_date) DESC, rowid DESC", 77);
-		case SORT_BY_ALPHABET_ASC:   return catas(q, " ORDER BY title ASC, rowid ASC", 30);
-		case SORT_BY_ALPHABET_DESC:  return catas(q, " ORDER BY title DESC, rowid DESC", 32);
+		case SORT_BY_TIME_ASC:       catas(q, " ORDER BY MAX(publication_date, update_date) ASC, rowid ASC", 59); break;
+		case SORT_BY_TIME_DESC:      catas(q, " ORDER BY MAX(publication_date, update_date) DESC, rowid DESC", 61); break;
+		case SORT_BY_ROWID_ASC:      catas(q, " ORDER BY rowid ASC", 19); break;
+		case SORT_BY_ROWID_DESC:     catas(q, " ORDER BY rowid DESC", 20); break;
+		case SORT_BY_UNREAD_ASC:     catas(q, " ORDER BY unread ASC, MAX(publication_date, update_date) DESC, rowid DESC", 73); break;
+		case SORT_BY_UNREAD_DESC:    catas(q, " ORDER BY unread DESC, MAX(publication_date, update_date) DESC, rowid DESC", 74); break;
+		case SORT_BY_IMPORTANT_ASC:  catas(q, " ORDER BY important ASC, MAX(publication_date, update_date) DESC, rowid DESC", 76); break;
+		case SORT_BY_IMPORTANT_DESC: catas(q, " ORDER BY important DESC, MAX(publication_date, update_date) DESC, rowid DESC", 77); break;
+		case SORT_BY_ALPHABET_ASC:   catas(q, " ORDER BY title ASC, rowid ASC", 30); break;
+		case SORT_BY_ALPHABET_DESC:  catas(q, " ORDER BY title DESC, rowid DESC", 32); break;
+		default:
+			fail_status("Unknown sorting method name!");
+			return false;
 	}
-	fail_status("Unknown sorting method name!");
-	return false;
+	return true;
 }
 
 static inline struct string *
-generate_search_query_string(const struct items_list *items, const struct string *search_filter)
+generate_search_query_string(const struct items_list *items)
 {
-	struct string *q = crtas("SELECT rowid,feed_url,title,link,publication_date,update_date,unread,important FROM items WHERE (", 97);
-	if (q == NULL) {
-		goto error;
+	struct string *query = crtas("SELECT rowid,feed_url,guid,title,link,publication_date,update_date,unread,important FROM items WHERE ", 101);
+	struct string *cond = generate_items_search_condition(items->feeds, items->feeds_count);
+	if (!STRING_IS_EMPTY(cond)) {
+		catss(query, cond);
 	}
-	for (size_t i = 0; i < items->feeds_count; ++i) {
-		if (i > 0 && !catas(q, " OR ", 4)) {
-			goto error;
-		}
-		if (!catas(q, "feed_url=?", 10)) {
-			goto error;
-		}
-		const struct string *item_rule = get_cfg_string(&items->feeds[i]->cfg, CFG_ITEM_RULE);
-		if (item_rule != NULL && item_rule->len > 0) {
-			if (!catas(q, " AND (", 6) || !catss(q, item_rule) || !catcs(q, ')')) {
-				goto error;
-			}
-		}
+	free_string(cond);
+	if (!STRING_IS_EMPTY(items->find_filter)) {
+		catas(query, " AND (", 6);
+		catss(query, items->find_filter);
+		catcs(query, ')');
 	}
-	if (!catcs(q, ')')) {
-		goto error;
+	if (!STRING_IS_EMPTY(items->search_filter)) {
+		catas(query, " AND ((title LIKE '%' || ? || '%') OR (content LIKE '%' || ? || '%'))", 69);
 	}
-	if (search_filter != NULL && search_filter->len > 0) {
-		if (!catas(q, " AND ((title LIKE '%' || ? || '%') OR (content LIKE '%' || ? || '%'))", 69)) {
-			goto error;
-		}
+	if (append_sorting_order_expression_to_query(query, items->sorting) == false) {
+		free_string(query);
+		return NULL;
 	}
-	if (append_sorting_order_expression_to_query(q, items->sorting) == false) {
-		goto error;
-	}
-	return q;
-error:
-	FAIL("Not enough memory for query string!");
-	free_string(q);
-	return NULL;
+	return query;
 }
 
 void
@@ -65,6 +52,7 @@ free_items_list(struct items_list *items)
 {
 	if (items != NULL) {
 		for (size_t i = 0; i < items->len; ++i) {
+			free_string(items->ptr[i].guid);
 			free_string(items->ptr[i].title);
 			free_string(items->ptr[i].url);
 			free_string(items->ptr[i].date_str);
@@ -72,6 +60,7 @@ free_items_list(struct items_list *items)
 		}
 		sqlite3_finalize(items->res);
 		free_string(items->query);
+		free_string(items->find_filter);
 		free_string(items->search_filter);
 		free(items->ptr);
 		free(items);
@@ -97,7 +86,6 @@ obtain_items_at_least_up_to_the_given_index(struct items_list *items, size_t ind
 	if (items->finished == true) {
 		return;
 	}
-	void *tmp;
 	const char *text;
 	int status;
 	while (index >= items->len) {
@@ -106,11 +94,8 @@ obtain_items_at_least_up_to_the_given_index(struct items_list *items, size_t ind
 			items->finished = true;
 			return;
 		}
-		tmp = realloc(items->ptr, sizeof(struct item_entry) * (items->len + 1));
-		if (tmp == NULL) {
-			return;
-		}
-		items->ptr = tmp;
+
+		items->ptr = newsraft_realloc(items->ptr, sizeof(struct item_entry) * (items->len + 1));
 
 		items->ptr[items->len].rowid = sqlite3_column_int64(items->res, 0);
 
@@ -121,10 +106,16 @@ obtain_items_at_least_up_to_the_given_index(struct items_list *items, size_t ind
 		}
 
 		text = (const char *)sqlite3_column_text(items->res, 2);
+		if (text == NULL) {
+			continue;
+		}
+		items->ptr[items->len].guid = crtas(text, strlen(text));
+
+		text = (const char *)sqlite3_column_text(items->res, 3);
 		items->ptr[items->len].title = text == NULL ? crtes(1) : crtas(text, strlen(text));
 		inlinefy_string(items->ptr[items->len].title);
 
-		text = (const char *)sqlite3_column_text(items->res, 3);
+		text = (const char *)sqlite3_column_text(items->res, 4);
 		items->ptr[items->len].url = crtes(1);
 		// Convert URL to absolute notation in case it's stored relative.
 		// For example, convert "/index.xml" to "http://example.org/index.xml"
@@ -134,8 +125,8 @@ obtain_items_at_least_up_to_the_given_index(struct items_list *items, size_t ind
 			free(full_url);
 		}
 
-		items->ptr[items->len].pub_date = sqlite3_column_int64(items->res, 4);
-		items->ptr[items->len].upd_date = sqlite3_column_int64(items->res, 5);
+		items->ptr[items->len].pub_date = sqlite3_column_int64(items->res, 5);
+		items->ptr[items->len].upd_date = sqlite3_column_int64(items->res, 6);
 		if (items->ptr[items->len].pub_date <= 0 && items->ptr[items->len].upd_date > 0) {
 			items->ptr[items->len].pub_date = items->ptr[items->len].upd_date;
 		}
@@ -145,55 +136,67 @@ obtain_items_at_least_up_to_the_given_index(struct items_list *items, size_t ind
 		items->ptr[items->len].date_str = get_cfg_date(NULL, CFG_LIST_ENTRY_DATE_FORMAT, items->ptr[items->len].upd_date);
 		items->ptr[items->len].pub_date_str = get_cfg_date(NULL, CFG_LIST_ENTRY_DATE_FORMAT, items->ptr[items->len].pub_date);
 
-		items->ptr[items->len].is_unread = sqlite3_column_int(items->res, 6); // unread
-		items->ptr[items->len].is_important = sqlite3_column_int(items->res, 7); // important
+		items->ptr[items->len].is_unread = sqlite3_column_int(items->res, 7); // unread
+		items->ptr[items->len].is_important = sqlite3_column_int(items->res, 8); // important
 
 		items->len += 1;
 	}
 }
 
 struct items_list *
-create_items_list(struct feed_entry **feeds, size_t feeds_count, int sorting, const struct wstring *search_filter)
+create_items_list(struct feed_entry **feeds, size_t feeds_count, const struct wstring *find_filter, const struct items_list *previous_items)
 {
 	INFO("Generating items list.");
 	if (feeds_count == 0) {
 		return NULL;
 	}
-	struct items_list *items = calloc(1, sizeof(struct items_list));
-	if (items == NULL) {
-		return NULL;
+
+	struct items_list *items = newsraft_calloc(1, sizeof(struct items_list));
+
+	if (previous_items) {
+		items->sorting = previous_items->sorting;
+	} else {
+		items->sorting = get_sorting_id(get_cfg_string(NULL, CFG_MENU_ITEM_SORTING)->ptr);
 	}
 
-	items->sorting = sorting;
-	if (sorting < 0) {
-		items->sorting = get_sorting_id(get_cfg_string(NULL, CFG_MENU_ITEM_SORTING)->ptr);
+	if (previous_items && !STRING_IS_EMPTY(previous_items->find_filter)) {
+		cpyss(&items->find_filter, previous_items->find_filter);
+	} else if (find_filter) {
+		items->find_filter = convert_wstring_to_string(find_filter);
+	}
+
+	if (previous_items && !STRING_IS_EMPTY(previous_items->search_filter)) {
+		cpyss(&items->search_filter, previous_items->search_filter);
+	} else {
+		items->search_filter = pop_search_filter();
 	}
 
 	items->feeds = feeds;
 	items->feeds_count = feeds_count;
-	items->search_filter = search_filter == NULL ? NULL : convert_wstring_to_string(search_filter);
-	items->query = generate_search_query_string(items, items->search_filter);
+	items->query = generate_search_query_string(items);
 	if (items->query == NULL) {
 		goto undo1;
 	}
-	items->res = db_prepare(items->query->ptr, items->query->len + 1);
+	items->res = db_prepare(items->query->ptr, items->query->len + 1, NULL);
 	if (items->res == NULL) {
-		fail_status("Can't run items search query! Make sure the item-rule setting is set correctly.");
+		fail_status("Can't run items search query! Make sure all item-rule settings are valid SQL conditions.");
 		goto undo2;
 	}
 	for (size_t i = 0; i < feeds_count; ++i) {
 		db_bind_string(items->res, i + 1, feeds[i]->link);
 	}
-	if (items->search_filter != NULL && items->search_filter->len > 0) {
+	if (!STRING_IS_EMPTY(items->search_filter)) {
 		db_bind_string(items->res, feeds_count + 1, items->search_filter);
 		db_bind_string(items->res, feeds_count + 2, items->search_filter);
 	}
 	obtain_items_at_least_up_to_the_given_index(items, 0);
 	if (items->len < 1) {
-		if (items->search_filter != NULL && items->search_filter->len > 0) {
-			info_status("Items not found. Search query didn't get any matches!");
+		if (!STRING_IS_EMPTY(items->search_filter)) {
+			info_status("No items found. Search query didn't get any matches!");
+		} else if (!STRING_IS_EMPTY(items->find_filter)) {
+			info_status("No items found. Find query didn't get any matches!");
 		} else {
-			fail_status("Items not found. Make sure this feed is updated!");
+			fail_status("No items found. Make sure this feed is updated!");
 		}
 		goto undo3;
 	}
@@ -203,6 +206,7 @@ undo3:
 undo2:
 	free_string(items->query);
 undo1:
+	free_string(items->find_filter);
 	free_string(items->search_filter);
 	free(items);
 	return NULL;
@@ -211,9 +215,7 @@ undo1:
 bool
 recreate_items_list(struct items_list **items)
 {
-	struct wstring *filter = (*items)->search_filter ? convert_string_to_wstring((*items)->search_filter) : NULL;
-	struct items_list *new_items = create_items_list((*items)->feeds, (*items)->feeds_count, (*items)->sorting, filter);
-	free_wstring(filter);
+	struct items_list *new_items = create_items_list((*items)->feeds, (*items)->feeds_count, NULL, *items);
 	if (new_items == NULL) {
 		return false;
 	}
@@ -248,16 +250,3 @@ change_items_list_sorting(struct items_list **items, input_id cmd)
 	recreate_items_list(items);
 	info_status(get_sorting_message((*items)->sorting), "items");
 }
-
-void
-change_search_filter_of_items_list(struct items_list **items, const struct wstring *search_filter)
-{
-	struct string *prev_search_filter = (*items)->search_filter;
-	(*items)->search_filter = convert_wstring_to_string(search_filter);
-	if (recreate_items_list(items) == true) {
-		free_string(prev_search_filter);
-	} else {
-		free_string((*items)->search_filter);
-		(*items)->search_filter = prev_search_filter;
-	}
-}
