@@ -67,6 +67,29 @@ important_item_condition(struct menu_state *ctx, size_t index)
 	return ctx->items->ptr[index].is_important;
 }
 
+struct string *
+generate_items_search_condition(struct feed_entry **feeds, size_t feeds_count)
+{
+	if (feeds_count == 0) {
+		return NULL;
+	}
+	struct string *cond = crtas("(", 1);
+	for (size_t i = 0; i < feeds_count; ++i) {
+		if (i > 0) {
+			catas(cond, " OR ", 4);
+		}
+		catas(cond, "feed_url=?", 10);
+		const struct string *rule = get_cfg_string(&feeds[i]->cfg, CFG_ITEM_RULE);
+		if (!STRING_IS_EMPTY(rule)) {
+			catas(cond, " AND (", 6);
+			catss(cond, rule);
+			catcs(cond, ')');
+		}
+	}
+	catcs(cond, ')');
+	return cond;
+}
+
 static void
 mark_item_read(struct menu_state *ctx, size_t view_sel, bool status)
 {
@@ -130,13 +153,13 @@ items_menu_loop(struct menu_state *m)
 	items_age += 1;
 	if (m->is_initialized == false) {
 		m->items_age = items_age;
-		m->items = create_items_list(m->feeds_original, m->feeds_count, -1, m->flags & MENU_USE_SEARCH ? search_mode_text_input : NULL);
+		m->items = create_items_list(m->feeds_original, m->feeds_count, m->find_filter, NULL);
 		if (m->items == NULL) {
 			return close_menu(); // Error displayed by create_items_list
 		}
 	}
 	start_menu();
-	const struct wstring *macro, *browser;
+	const struct wstring *arg, *browser;
 	while (true) {
 		if (get_cfg_bool(NULL, CFG_MENU_RESPONSIVENESS) && m->items_age != items_age) {
 			m->items_age = items_age;
@@ -145,8 +168,8 @@ items_menu_loop(struct menu_state *m)
 		if (get_cfg_bool(&m->items->ptr[m->view_sel].feed[0]->cfg, CFG_MARK_ITEM_READ_ON_HOVER)) {
 			mark_item_read(m, m->view_sel, true);
 		}
-		input_id cmd = get_input(m->items->ptr[m->view_sel].feed[0]->binds, NULL, &macro);
-		if (handle_list_menu_control(m, cmd, macro) == true) {
+		input_id cmd = get_input(m->items->ptr[m->view_sel].feed[0]->binds, NULL, &arg);
+		if (handle_list_menu_control(m, cmd, arg) == true) {
 			continue;
 		}
 		switch (cmd) {
@@ -161,19 +184,20 @@ items_menu_loop(struct menu_state *m)
 			case INPUT_COPY_TO_CLIPBOARD: copy_string_to_clipboard(m->items->ptr[m->view_sel].url); break;
 			case INPUT_QUIT_HARD:         return NULL;
 			case INPUT_NAVIGATE_BACK:
-				if (get_menu_depth() < 3 && (m->flags & MENU_IS_EXPLORE)) break;
+				if (get_menu_depth() < 3 && (m->flags & MENU_IS_EXPLORE && m->find_filter == NULL)) break;
 				// fall through
 			case INPUT_QUIT_SOFT:
-				if (m->flags & MENU_IS_EXPLORE) close_menu();
+				if (m->flags & MENU_IS_EXPLORE && m->find_filter == NULL) close_menu();
 				return close_menu();
 			case INPUT_TOGGLE_EXPLORE_MODE:
 				if (m->flags & MENU_IS_EXPLORE) return close_menu();
 				break;
 			case INPUT_GOTO_FEED:
 				if (!(m->flags & MENU_IS_EXPLORE)) break;
-				return setup_menu(&items_menu_loop, NULL, m->items->ptr[m->view_sel].feed, 1, MENU_NORMAL);
+				return setup_menu(&items_menu_loop, NULL, m->items->ptr[m->view_sel].feed, 1, MENU_NORMAL, NULL);
 			case INPUT_APPLY_SEARCH_MODE_FILTER:
-				change_search_filter_of_items_list(&m->items, search_mode_text_input); break;
+				cpyas(&m->items->search_filter, "", 0);
+				recreate_items_list(&m->items); break;
 			case INPUT_OPEN_IN_BROWSER:
 				browser = get_cfg_wstring(&m->items->ptr[m->view_sel].feed[0]->cfg, CFG_OPEN_IN_BROWSER_COMMAND);
 				run_formatted_command(browser, get_item_args(m, m->view_sel));
@@ -184,8 +208,14 @@ items_menu_loop(struct menu_state *m)
 			case INPUT_SORT_BY_ALPHABET:
 			case INPUT_SORT_BY_IMPORTANT:
 				change_items_list_sorting(&m->items, cmd); break;
+			case INPUT_FIND_COMMAND:
+				if (m->find_filter) return setup_menu(&items_menu_loop, NULL, m->feeds_original, m->feeds_count, MENU_IS_EXPLORE | MENU_SWALLOW, arg);;
+				return setup_menu(&items_menu_loop, NULL, m->feeds_original, m->feeds_count, MENU_IS_EXPLORE, arg);
+			case INPUT_DATABASE_COMMAND:
+				db_perform_user_edit(arg, NULL, 0, &m->items->ptr[m->view_sel]);
+				break;
 			case INPUT_ENTER:
-				return setup_menu(&item_pager_loop, m->items->ptr[m->view_sel].title, NULL, 0, MENU_NORMAL);
+				return setup_menu(&item_pager_loop, m->items->ptr[m->view_sel].title, NULL, 0, MENU_NORMAL, NULL);
 		}
 	}
 	return close_menu();
