@@ -11,10 +11,9 @@
 #include <curses.h>
 #include <expat.h>
 #include <curl/curl.h>
-#include <yajl/yajl_parse.h>
 
 #ifndef NEWSRAFT_VERSION
-#define NEWSRAFT_VERSION "0.29"
+#define NEWSRAFT_VERSION "0.30"
 #endif
 
 #define ISWHITESPACE(A) (((A)==' ')||((A)=='\n')||((A)=='\t')||((A)=='\v')||((A)=='\f')||((A)=='\r'))
@@ -79,9 +78,8 @@ enum {
 enum {
 	MENU_NORMAL           = 0,
 	MENU_IS_EXPLORE       = 1,  // Tell items menu to use explore mode
-	MENU_USE_SEARCH       = 2,  // Tell items menu to apply search
-	MENU_SWALLOW          = 4,  // Replace current menu with this menu
-	MENU_DISABLE_SETTINGS = 8,
+	MENU_SWALLOW          = 2,  // Replace current menu with this menu
+	MENU_DISABLE_SETTINGS = 4,
 };
 
 typedef uint8_t sorting_method_t;
@@ -144,7 +142,7 @@ struct wstring {
 
 struct binding_action {
 	input_id cmd;
-	struct wstring *exec;
+	struct wstring *arg;
 };
 
 struct input_binding {
@@ -161,11 +159,11 @@ struct feed_entry {
 	int64_t update_date; // Date of last feed update attempt
 	struct config_context *cfg;
 	struct input_binding *binds;
-	bool has_errors;
 	struct string *errors;
 };
 
 struct item_entry {
+	struct string *guid;
 	struct string *title;
 	struct string *url;
 	struct feed_entry **feed;
@@ -181,6 +179,7 @@ struct item_entry {
 struct items_list {
 	sqlite3_stmt *res;
 	struct string *query;
+	struct string *find_filter;
 	struct string *search_filter;
 	struct item_entry *ptr;
 	size_t len;
@@ -214,11 +213,13 @@ struct menu_state {
 	bool is_initialized;
 	bool is_deleted;
 	const struct wstring *entry_format;
+	const struct wstring *find_filter;
 	bool (*enumerator)(struct menu_state *ctx, size_t index); // Checks if index is valid
 	const struct format_arg *(*get_args)(struct menu_state *ctx, size_t index);
 	unsigned (*paint_action)(struct menu_state *ctx, size_t index);
 	void (*write_action)(size_t index, WINDOW *w);
 	bool (*unread_state)(struct menu_state *ctx, size_t index);
+	bool (*failed_state)(struct menu_state *ctx, size_t index);
 	struct menu_state *prev;
 };
 
@@ -288,6 +289,7 @@ struct getfeed_feed {
 
 struct feed_update_state {
 	struct feed_entry *feed_entry;
+	struct string *new_errors;
 
 	CURL *curl;
 	char curl_error[CURL_ERROR_SIZE];
@@ -296,7 +298,6 @@ struct feed_update_state {
 
 	int8_t media_type;
 	XML_Parser xml_parser;
-	yajl_handle json_parser;
 	struct getfeed_feed feed;
 	bool in_item;
 	struct string *text;
@@ -329,9 +330,7 @@ bool start_updating_all_feeds_and_wait_finish(void);
 bool print_unread_items_count(void);
 void process_auto_updating_feeds(void);
 void mark_feeds_read(struct feed_entry **feeds, size_t feeds_count, bool status);
-#ifdef TEST
 struct feed_entry **get_all_feeds(size_t *feeds_count);
-#endif
 
 // See "feeds-parse.c" file for implementation.
 bool parse_feeds_file(void);
@@ -352,7 +351,7 @@ bool handle_list_menu_control(struct menu_state *m, input_id cmd, const struct w
 bool handle_pager_menu_control(input_id cmd);
 void free_menus(void);
 size_t get_menu_depth(void);
-struct menu_state *setup_menu(struct menu_state *(*run)(struct menu_state *), const struct string *name, struct feed_entry **feeds, size_t feeds_count, uint32_t flags);
+struct menu_state *setup_menu(struct menu_state *(*run)(struct menu_state *), const struct string *name, struct feed_entry **feeds, size_t feeds_count, uint32_t flags, const void *ctx);
 struct menu_state *close_menu(void);
 void start_menu(void);
 void write_menu_path_string(struct string *names, struct menu_state *m);
@@ -371,16 +370,16 @@ int get_sorting_id(const char *sorting_name);
 const char *get_sorting_message(int sorting_id);
 
 // See "items.c" file for implementation.
+struct string *generate_items_search_condition(struct feed_entry **feeds, size_t feeds_count);
 bool important_item_condition(struct menu_state *ctx, size_t index);
 void tell_items_menu_to_regenerate(void);
 struct menu_state *items_menu_loop(struct menu_state *dest);
 
 // See "items-list.c" file for implementation.
-struct items_list *create_items_list(struct feed_entry **feeds, size_t feeds_count, int sorting, const struct wstring *search_filter);
+struct items_list *create_items_list(struct feed_entry **feeds, size_t feeds_count, const struct wstring *find_filter, const struct items_list *previous_items);
 bool recreate_items_list(struct items_list **items);
 void obtain_items_at_least_up_to_the_given_index(struct items_list *items, size_t index);
 void change_items_list_sorting(struct items_list **items, input_id cmd);
-void change_search_filter_of_items_list(struct items_list **items, const struct wstring *search_filter);
 void free_items_list(struct items_list *items);
 
 // See "items-pager.c" file for implementation.
@@ -426,15 +425,17 @@ const char *get_db_path(void);
 // See "dates.c" file for implementation.
 int64_t parse_date(const char *str, bool rfc3339_first);
 struct string *get_cfg_date(struct config_context **ctx, config_entry_id format_id, int64_t date);
+struct timespec newsraft_get_monotonic_time(void);
+struct string *newsraft_get_pretty_time_diff(struct timespec *start, struct timespec *stop);
 
 // See "db.c" file for implementation.
 bool db_init(void);
 bool db_vacuum(void);
 bool exec_database_file_optimization(void);
 void db_stop(void);
-sqlite3_stmt *db_prepare(const char *zSql, int nByte);
-bool db_begin_transaction(void);
-bool db_commit_transaction(void);
+sqlite3_stmt *db_prepare(const char *statement, int size, const char **p_error);
+bool db_transaction_begin(void);
+bool db_transaction_commit(void);
 bool db_rollback_transaction(void);
 const char *db_error_string(void);
 int db_bind_string(sqlite3_stmt *stmt, int pos, const struct string *str);
@@ -442,12 +443,12 @@ int64_t db_get_date_from_feeds_table(const struct string *url, const char *colum
 struct string *db_get_string_from_feed_table(const struct string *url, const char *column, size_t column_len);
 void db_update_feed_int64(const struct string *url, const char *column_name, int64_t value, bool only_positive);
 void db_update_feed_string(const struct string *url, const char *column_name, const struct string *value, bool only_nonempty);
+void db_perform_user_edit(const struct wstring *fmt, struct feed_entry **feeds, size_t feeds_count, const struct item_entry *item);
 
 // See "db-items.c" file for implementation.
 sqlite3_stmt *db_find_item_by_rowid(int64_t rowid);
 bool db_mark_item_read(int64_t rowid, bool status);
 bool db_mark_item_important(int64_t rowid, bool status);
-int64_t get_unread_items_count_of_the_feed(const struct string *url);
 int64_t db_count_items(struct feed_entry **feeds, size_t feeds_count, bool count_only_unread);
 bool db_change_unread_status_of_all_items_in_feeds(struct feed_entry **feeds, size_t feeds_count, bool unread);
 
@@ -473,18 +474,18 @@ void prevent_status_cleaning(void);
 void allow_status_cleaning(void);
 void status_write(config_entry_id color, const char *format, ...);
 void status_delete(void);
-input_id get_input(struct input_binding *ctx, uint32_t *count, const struct wstring **macro_ptr);
+input_id get_input(struct input_binding *ctx, uint32_t *count, const struct wstring **p_arg);
 void break_getting_input_command(void);
+struct string *pop_search_filter(void);
 
 // See "interface-errors-pager.c" file for implementation.
 struct menu_state *errors_pager_loop(struct menu_state *m);
 
 // Functions responsible for managing of key bindings.
 // See "binds.c" file for implementation.
-input_id get_action_of_bind(struct input_binding *ctx, const char *key, size_t action_index, const struct wstring **macro_ptr);
+input_id get_action_of_bind(struct input_binding *ctx, const char *key, size_t action_index, const struct wstring **p_arg);
 struct input_binding *create_or_clean_bind(struct input_binding **target, const char *key);
-bool attach_action_to_bind(struct input_binding *bind, input_id action);
-bool attach_command_to_bind(struct input_binding *bind, const char *exec, size_t exec_len);
+bool attach_action_to_bind(struct input_binding *bind, input_id cmd, const char *arg, size_t arg_len);
 bool assign_default_binds(void);
 input_id get_input_id_by_name(const char *name);
 void free_binds(struct input_binding *target);
@@ -499,12 +500,12 @@ void run_formatted_command(const struct wstring *wcmd_fmt, const struct format_a
 struct string *crtes(size_t desired_capacity);
 struct string *crtas(const char *src_ptr, size_t src_len);
 struct string *crtss(const struct string *src);
-bool cpyas(struct string **dest, const char *src_ptr, size_t src_len);
-bool cpyss(struct string **dest, const struct string *src);
-bool catas(struct string *dest, const char *src_ptr, size_t src_len);
-bool catss(struct string *dest, const struct string *src);
-bool catcs(struct string *dest, char c);
-bool make_string_fit_more(struct string **dest, size_t n);
+void cpyas(struct string **dest, const char *src_ptr, size_t src_len);
+void cpyss(struct string **dest, const struct string *src);
+void catas(struct string *dest, const char *src_ptr, size_t src_len);
+void catss(struct string *dest, const struct string *src);
+void catcs(struct string *dest, char c);
+void make_string_fit_more(struct string **dest, size_t n);
 void str_vappendf(struct string *dest, const char *fmt, va_list args);
 void str_appendf(struct string *dest, const char *fmt, ...);
 void empty_string(struct string *dest);
@@ -514,23 +515,25 @@ struct wstring *convert_string_to_wstring(const struct string *src);
 struct wstring *convert_array_to_wstring(const char *src_ptr, size_t src_len);
 void remove_start_of_string(struct string *str, size_t size);
 void inlinefy_string(struct string *title);
+void newsraft_simple_hash(struct string **dest, const char *src);
+struct string *newsraft_base64_encode(const uint8_t *data, size_t size);
 
 // See "string-serialize.c" file for implementation.
-bool serialize_caret(struct string **target);
-bool serialize_array(register struct string **target, register const char *key, register size_t key_len, register const char *value, register size_t value_len);
-bool serialize_string(struct string **target, const char *key, size_t key_len, const struct string *value);
+void serialize_caret(struct string **target);
+void serialize_array(register struct string **target, register const char *key, register size_t key_len, register const char *value, register size_t value_len);
+void serialize_string(struct string **target, const char *key, size_t key_len, const struct string *value);
 struct deserialize_stream *open_deserialize_stream(const char *serialized_data);
 const struct string *get_next_entry_from_deserialize_stream(struct deserialize_stream *stream);
 void close_deserialize_stream(struct deserialize_stream *stream);
 
 // See "wstring.c" file for implementation.
-bool wstr_set(struct wstring **dest, const wchar_t *src_ptr, size_t src_len, size_t src_lim);
+void wstr_set(struct wstring **dest, const wchar_t *src_ptr, size_t src_len, size_t src_lim);
 struct wstring *wcrtes(size_t desired_capacity);
 struct wstring *wcrtas(const wchar_t *src_ptr, size_t src_len);
-bool wcatas(struct wstring *dest, const wchar_t *src_ptr, size_t src_len);
-bool wcatss(struct wstring *dest, const struct wstring *src);
-bool wcatcs(struct wstring *dest, wchar_t c);
-bool make_sure_there_is_enough_space_in_wstring(struct wstring *dest, size_t need_space);
+void wcatas(struct wstring *dest, const wchar_t *src_ptr, size_t src_len);
+void wcatss(struct wstring *dest, const struct wstring *src);
+void wcatcs(struct wstring *dest, wchar_t c);
+void make_sure_there_is_enough_space_in_wstring(struct wstring *dest, size_t need_space);
 void empty_wstring(struct wstring *dest);
 void free_wstring(struct wstring *wstr);
 struct string *convert_wstring_to_string(const struct wstring *src);
@@ -594,22 +597,29 @@ void threads_stop(void);
 
 // See "parse_xml" directory for implementation.
 bool setup_xml_parser(struct feed_update_state *data);
+bool convert_opml_to_feeds(void);
+bool convert_feeds_to_opml(void);
 
 // See "parse_json" directory for implementation.
-bool setup_json_parser(struct feed_update_state *data);
+void setup_json_parser(struct feed_update_state *data);
+bool newsraft_json_parse(struct feed_update_state *data, const char *content, size_t content_size);
 
 // See "insert_feed" directory for implementation.
 bool insert_feed(struct feed_entry *feed, struct getfeed_feed *feed_data);
 
 // See "struct-item.c" file for implementation.
-bool prepend_item(struct getfeed_item **head_item_ptr);
+void prepend_item(struct getfeed_item **head_item_ptr);
 void free_item(struct getfeed_item *item);
 
+// See "alloc.c" file for implementation.
+void *newsraft_malloc(size_t size);
+void *newsraft_calloc(size_t n, size_t size);
+void *newsraft_realloc(void *ptr, size_t size);
+void newsraft_free(void *ptr);
+
 extern volatile bool they_want_us_to_stop;
 extern size_t list_menu_height;
 extern size_t list_menu_width;
-extern bool search_mode_is_enabled;
-extern struct wstring *search_mode_text_input;
 extern pthread_mutex_t interface_lock;
 
 #include "config.h"
